
def method(*args, **kwargs):
    return lambda x: x


class BusName(object):
    def __init__(self, *args, **kwargs):
        pass


class Object:
    def __init__(self, *args, **kwargs):
        pass
